'use client';

import { FormEvent, MouseEvent, useState, useEffect } from 'react';
import Link from 'next/link';
import { Album } from '@/entities/album.entity';
import { createAlbum, getAlbum, getAlbums } from '@/app/actions';
import Button from 'react-bootstrap/Button';
import Modal from 'react-bootstrap/Modal';
import { IconButton } from '@mui/material';
import { AddCircleOutline, Delete, Edit } from '@mui/icons-material';

export default function Page() {
  const [albums, setAlbums] = useState<Album[]>([]);
  const [show, setShow] = useState(false);
  const handleClose = () => setShow(false);
  const handleShow = () => setShow(true);
  const [formAlbumTitle, setFormAlbumTitle] = useState("");
  const [formAlbumArtist, setFormAlbumArtist] = useState("");
  const [formAlbumGenre, setFormAlbumGenre] = useState("");

  useEffect(() => {
    async function fetchAlbums() {
      try {
        const response = await fetch('https://api.anatid.net/album');
        const data = await response.json();
        setAlbums(data);
      } catch (error) {
        console.error("Error getting albums: ", error);
      }
    }
    fetchAlbums();
  });

  const handleSubmit = async (event: FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    handleClose();

    const formData = new FormData(event.currentTarget);
    try {
      const response = await createAlbum(formData);
      const data = await response.json();
      console.log(data);
    } catch (error) {
      console.error("Error creating Album: ", error);
    }
  }

  const handleCreate = async () => {
    setFormAlbumTitle("");
    setFormAlbumArtist("");
    setFormAlbumGenre("");
    handleShow();
  }

  const handleEdit = async (event: MouseEvent<HTMLElement>) => {
    const id = event.currentTarget.getAttribute('id');
    if (id) {
      try {
        const response = await getAlbum(parseInt(id));
        const data = await response.json();
        setFormAlbumTitle(data.title);
        setFormAlbumArtist(data.artist);
        setFormAlbumGenre(data.genre);
        handleShow();
      } catch (error) {
        console.error(`Error getting album with ID ${id}:`, error);
      }
    } else {
      console.error("Couldn't get ID of clicked element");
    }
  }

  if (!albums) return <div>Loading...</div>

  return (
    <>
    <div className="container">
      <IconButton aria-label="Add Album" size="large" onClick={handleCreate}>
        <AddCircleOutline fontSize="inherit" color="success"/>
      </IconButton>
      <table className="u-full-width">
        <thead>
          <tr>
            <th>Title</th>
            <th>Artist</th>
            <th>Genre</th>
            <th>Controls</th>
          </tr>
        </thead>
        <tbody>
        {albums.map((album: Album) => (
          <tr key={album.id}>
            <td>
              <Link
                href={{
                  pathname: `/album/${album.id}`
                }}>{album.title}
              </Link>
            </td>
            <td>
              {album.artist}
            </td>
            <td>
              {album.genre}
            </td>
            <td>
              <IconButton aria-label="Edit Album" size="small" id={album.id.toString()} onClick={handleEdit}>
                <Edit fontSize="inherit" color="success"/>
              </IconButton>
              <IconButton aria-label="Delete Album" size="small" id={album.id.toString()} onClick={handleEdit}>
                <Delete fontSize="inherit" color="success"/>
              </IconButton>
            </td>
          </tr>
        ))}
        </tbody>
      </table>

    </div>

    <Modal
      show={show}
      onHide={handleClose}
      backdrop="static"
      keyboard={false}
    >
      <Modal.Header closeButton>
        <Modal.Title>Create Album</Modal.Title>
      </Modal.Header>
      <Modal.Body>
        <div>
          <form onSubmit={handleSubmit}>
            <div className="row">
              <div className="six columns">
                <label htmlFor="album-title">Album Title</label><input type="text" name="title" id="album-title" defaultValue={formAlbumTitle} />
              </div>
              <div className="six columns">
                <label htmlFor="album-artist">Artist</label><input type="text" name="artist" id="album-artist" defaultValue={formAlbumArtist} />
              </div>
            </div>
            <div className="row">
              <div className="six columns">
                <label htmlFor="album-genre">Genre</label><input type="text" name="genre" id="album-genre" defaultValue={formAlbumGenre} />
              </div>
            </div>
            <Button variant="primary" type="submit">Create</Button>
          </form>
        </div>
      </Modal.Body>
      <Modal.Footer>
        <Button variant="secondary" onClick={handleClose}>
          Close
        </Button>
      </Modal.Footer>
    </Modal>
    </>
  );
}
